import { createSignal } from "solid-js";
import { api, Group } from "./api";
import { showDialog, showInputDialog, showMessageDialog } from "./state";
import { createFileUploader } from "@solid-primitives/upload";

export function errorToString(e: any): string { // eslint-disable-line @typescript-eslint/no-explicit-any
	if (typeof e == 'string') {
		return e;
	} else if (e.toString) {
		return e.toString() as string;
	} else {
		return 'Unknown error';
	}
}

export function normalize(text: string, maxLength: number, oneLine: boolean) {
	if (oneLine) text = text.replaceAll('\n', '');
	text = text.substring(0, maxLength);
	return text;
}

export function normalizeQuantity(text: string) {
	let quantity = parseInt(text);

	if (isNaN(quantity) || quantity < 0) {
		return 1;
	}

	return quantity;
}

function clamp(value: number, min: number, max: number) {
	return Math.min(Math.max(value, min), max);
};

export function normalizeRating(text: string) {
	let rating = parseInt(text);

	if (isNaN(rating)) {
		return 1;
	}

	return clamp(rating, 0, 5);
}

export function _arrayBufferToBase64(buffer: ArrayBuffer) {
	var binary = '';
	var bytes = new Uint8Array(buffer);
	var len = bytes.byteLength;
	for (var i = 0; i < len; i++) {
		binary += String.fromCharCode(bytes[i]);
	}
	return window.btoa(binary);
}

export const b64toBlob = (b64Data: string, contentType = '', sliceSize = 512) => {
	const byteCharacters = atob(b64Data);
	const byteArrays = [];

	for (let offset = 0; offset < byteCharacters.length; offset += sliceSize) {
		const slice = byteCharacters.slice(offset, offset + sliceSize);

		const byteNumbers = new Array(slice.length);
		for (let i = 0; i < slice.length; i++) {
			byteNumbers[i] = slice.charCodeAt(i);
		}

		const byteArray = new Uint8Array(byteNumbers);
		byteArrays.push(byteArray);
	}

	const blob = new Blob(byteArrays, { type: contentType });
	return blob;
}

export const addMember = (group: Group, onClose: (newGroup: Group) => void) => {
	showInputDialog('Add member to Group', 'enter the member id you want to add', 'member ID', async (memberId) => {
		try {
			onClose(await api().addMember(group.id, memberId));
		} catch (e) {
			showMessageDialog('Failed to add member', errorToString(e));
		}
	})
};

export const addStratType = (group: Group, onClose: (newGroup: Group) => void) => {
	showInputDialog('Add strat type to Group', 'enter the name you want to add', 'strat type', async (stratType) => {
		try {
			onClose(await api().addStratType(group.id, stratType));
		} catch (e) {
			showMessageDialog('Failed to add strat type', errorToString(e));
		}
	})
};

export const addProfile = (group: Group, onClose: (newGroup: Group) => void) => {
	const [profileName, setProfileName] = createSignal('');
	const { files, selectFiles } = createFileUploader({
		multiple: false,
		accept: "image/*",
	});

	let fileReader = new FileReader();

	fileReader.onload = async event => {
		try {
			onClose(await api().addProfileToGroup(group.id, { name: profileName(), image: _arrayBufferToBase64(event.target!.result as ArrayBuffer) }));
		} catch (e) {
			showMessageDialog('Failed to add profile', errorToString(e));
		}
	}

	showDialog({
		title: 'Add profile to Group',
		text: 'create a new profile',
		inputFields: [{
			onInput: setProfileName,
			placeholder: 'name',
			name: 'name'
		}],
		onDismiss: async () => {
			fileReader.readAsArrayBuffer(files()[0].file);
		},
		content: <button onclick={() => selectFiles(([{ source, name, size, file }]) => {
			console.log({ source, name, size, file });
		})}> Add Image </button>,
		buttons: ['confirm'],
	});
};

export const addMap = async (group: Group, onClose: (newGroup: Group) => void) => {
	const [mapName, setMapName] = createSignal('');
	const { files, selectFiles } = createFileUploader({
		multiple: false,
		accept: "image/*",
	});

	let fileReader = new FileReader();

	fileReader.onload = async event => {
		try {
			onClose(await api().addMap(group.id, { name: mapName(), image: _arrayBufferToBase64(event.target!.result as ArrayBuffer) }));
		} catch (e) {
			showMessageDialog('Failed to add map', errorToString(e));
		}
	}

	showDialog({
		title: 'Add map to Group',
		text: 'create a new map',
		inputFields: [{
			onInput: setMapName,
			placeholder: 'name',
			name: 'name'
		}],
		onDismiss: async () => {
			fileReader.readAsArrayBuffer(files()[0].file);
		},
		content: <button onclick={() => selectFiles(([{ source, name, size, file }]) => {
			console.log({ source, name, size, file });
		})}> Add Image </button>,
		buttons: ['confirm'],
	});
};